#include <stdlib.h>
#ifdef __APPLE__
#include <GLUT/glut.h>
#elif defined(WIN32)
#include <glut.h>
#else
#include <GL/glut.h>
#endif

#include "mesh.h"
#include "objfile.h"

/* use objfile to load mesh data (vertices, normals, and texcoords)
 * from a wavefront|OBJ file.
 */
struct mesh *load_mesh(const char *fname)
{
	int i;
	struct mesh *m;
	struct objfile *obj;

	if(!(obj = objf_load(fname))) {
		return 0;
	}
	m = malloc(sizeof *m);
	m->num_verts = objf_vertex_count(obj);

	m->vert = malloc(m->num_verts * sizeof *m->vert);
	m->norm = malloc(m->num_verts * sizeof *m->norm);
	m->tc = malloc(m->num_verts * sizeof *m->tc);

	for(i=0; i<m->num_verts; i++) {
		float *v = objf_vertex(obj, i);
		float *n = objf_normal(obj, i);
		float *t = objf_texcoord(obj, i);

		m->vert[i].x = v[0];
		m->vert[i].y = v[1];
		m->vert[i].z = v[2];

		m->norm[i].x = n[0];
		m->norm[i].y = n[1];
		m->norm[i].z = n[2];

		m->tc[i].x = t[0];
		m->tc[i].y = t[1];
	}
	objf_free(obj);
	return m;
}

void free_mesh(struct mesh *m)
{
	if(m) {
		free(m->vert);
		free(m->norm);
		free(m->tc);
		free(m);
	}
}

void render_mesh(struct mesh *m)
{
	int i;

	glBegin(GL_TRIANGLES);

	for(i=0; i<m->num_verts; i++) {
		glNormal3f(m->norm[i].x, m->norm[i].y, m->norm[i].z);
		glTexCoord2f(m->tc[i].x, m->tc[i].y);
		glVertex3f(m->vert[i].x, m->vert[i].y, m->vert[i].z);
	}
	glEnd();
}
